//  
//   Rebex Sample Code License
// 
//   Copyright 2024, REBEX CR s.r.o.
//   All rights reserved.
//   https://www.rebex.net/
// 
//   Permission to use, copy, modify, and/or distribute this software for any
//   purpose with or without fee is hereby granted.
// 
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//   OTHER DEALINGS IN THE SOFTWARE.
// 

using System;
using System.ComponentModel;
using System.Windows.Forms;
using Rebex.TerminalEmulation;

namespace Rebex.Samples.Scripts
{
    [Description("06: Scripting conditions")]
    public class ScriptingConditions
    {
        public static void Execute(Scripting scripting)
        {
            // detect prompt for later use
            scripting.DetectPrompt();

            // start multiline echo command
            scripting.SendCommand(@"echo \");

            // see if we get ">" (= multiline echo is supported) or Prompt
            ScriptMatch match = scripting.WaitFor(">", ScriptEvent.Prompt);

            // equivalent to:
            // ScriptMatch match = scripting.WaitFor(">" | ScriptEvent.Prompt);
            // and to:
            // ScriptMatch match = scripting.WaitFor(ScriptEvent.Or(ScriptEvent.FromString(">"), ScriptEvent.Prompt));

            // did we get ">"?
            if (match.IsEventMatched(">"))
            {
                // end the command with Ctrl+C
                scripting.Send(ConsoleKey.C, ConsoleModifiers.Control);

                // wait if prompt appears
                match = scripting.WaitFor(ScriptEvent.Prompt | ScriptEvent.Delay(200));

                // if prompt does not appear
                if (!match.IsPrompt)
                {
                    // send something as the echo message
                    scripting.SendCommand("Server does not support Ctrl+C.");
                    
                    // and wait for prompt
                    scripting.WaitFor(ScriptEvent.Prompt);
                }
            }			

            // end of script
        }
    }
}
