//  
//   Rebex Sample Code License
// 
//   Copyright 2024, REBEX CR s.r.o.
//   All rights reserved.
//   https://www.rebex.net/
// 
//   Permission to use, copy, modify, and/or distribute this software for any
//   purpose with or without fee is hereby granted.
// 
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//   OTHER DEALINGS IN THE SOFTWARE.
// 

using System;
using Rebex.Net;
using Rebex;

namespace Rebex.Samples
{
    /// <summary>
    /// This sample connects to a WebSocket server, sends 'Hello!', receives a response and displays it.
    /// </summary>
    class Program
    {
        [STAThread]
        static void Main(string[] args)
        {
            // set the trial license key
            Rebex.Licensing.Key = Rebex.Samples.TrialKey.Key;


            // parse arguments
            string uri;
            if (args.Length == 0)
            {
                uri = "ws://echo.websocket.org";
            }
            else
            {
                uri = args[0];
                if (uri.StartsWith("-"))
                {
                    Console.WriteLine("Usage: WebSocketEcho [uri]");
                    Console.WriteLine("Example: WebSocketEcho ws://echo.websocket.org");
                    return;
                }
            }

            // create an instance of WebSocket client
            var client = new WebSocketClient();

            // log communication
            client.LogWriter = new ConsoleLogWriter(LogLevel.Info);

            // connect to a server
            client.Connect(uri);

            // send a message
            client.Send("Hello!");

            // receive response
            string response = client.Receive<string>();

            // display it
            Console.WriteLine("Response: {0}", response);
        }
    }
}
