//  
//   Rebex Sample Code License
// 
//   Copyright 2024, REBEX CR s.r.o.
//   All rights reserved.
//   https://www.rebex.net/
// 
//   Permission to use, copy, modify, and/or distribute this software for any
//   purpose with or without fee is hereby granted.
// 
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//   OTHER DEALINGS IN THE SOFTWARE.
// 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Rebex.Samples
{
    public static class ArraySegmentHelper
    {
        public static ArraySegment<byte> Slice(ArraySegment<byte> buffer, int offset, int count)
        {
            return new ArraySegment<byte>(buffer.Array, buffer.Offset + offset, count);
        }

        public static ArraySegment<byte> Slice(ArraySegment<byte> buffer, int count)
        {
            return new ArraySegment<byte>(buffer.Array, buffer.Offset + count, buffer.Count - count);
        }
    }

    public static class TaskHelpers
    {
        public static Action<Task> GetTaskLogErrorFunc(Action<LogLevel, string> logger, string message)
        {
            return t =>
            {
                if (t.IsFaulted)
                {
                    logger(LogLevel.Error, string.Format("{0}: {1}", message, t.Exception));
                }
            };
        }

        public static Action<Task> GetTaskLogSuccessFunc(Action<LogLevel, string> logger, LogLevel level, string message)
        {
            return t =>
            {
                if (t.IsCompleted && !t.IsFaulted && !t.IsCanceled)
                {
                    logger(level, message);
                }
            };
        }
    }
}
