//  
//   Rebex Sample Code License
// 
//   Copyright 2024, REBEX CR s.r.o.
//   All rights reserved.
//   https://www.rebex.net/
// 
//   Permission to use, copy, modify, and/or distribute this software for any
//   purpose with or without fee is hereby granted.
// 
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//   OTHER DEALINGS IN THE SOFTWARE.
// 

using System;
using System.Text;
using Rebex.Mail;
using Rebex.Mime;
using Rebex.Mime.Headers;

namespace Rebex.Samples
{
    /// <summary>
    /// Provides formatting and parsing functionality for <see cref="MsgAddress"/>.
    /// </summary>
    public static class MsgAddressFormatter
    {
        public static string ToString(MsgAddress address)
        {
            if (address.AddressType == MsgAddressType.Unknown)
                return string.Empty;
            else if (address.AddressType == MsgAddressType.Smtp)
                return new MailAddress(address.SmtpAddress, address.DisplayName).ToString();
            else
                return address.ToString();
        }

        public static string ToString(MsgAddressCollection addresses)
        {
            var sb = new StringBuilder();
            foreach (var address in addresses)
            {
                if (sb.Length > 0)
                    sb.Append(", ");
                sb.Append(ToString(address));
            }
            return sb.ToString();
        }

        public static MailAddressCollection ParseSmtpAddressCollection(string text)
        {
            if (string.IsNullOrEmpty(text))
                return null;

            try
            {
                var list = new MailAddressCollection(text);
                foreach (MailAddress address in list)
                {
                    if (string.IsNullOrEmpty(address.User) || string.IsNullOrEmpty(address.Host))
                        return null;
                }
                return list;
            }
            catch (MimeException x)
            {
                if (x.Status != MimeExceptionStatus.HeaderParserError)
                    throw;
                return null;
            }
        }
    }
}
