//  
//   Rebex Sample Code License
// 
//   Copyright 2024, REBEX CR s.r.o.
//   All rights reserved.
//   https://www.rebex.net/
// 
//   Permission to use, copy, modify, and/or distribute this software for any
//   purpose with or without fee is hereby granted.
// 
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//   OTHER DEALINGS IN THE SOFTWARE.
// 

using System;
using System.IO;
using Rebex.Net;

namespace Rebex.Samples
{
    /// <summary>
    /// This sample is a simple utility to upload/download a file to/from
    /// a FTP server using the Ftp class.
    /// </summary>
    class GetPut
    {

        [STAThread]
        static void Main(string[] args)
        {
            // set the trial license key
            Rebex.Licensing.Key = Rebex.Samples.TrialKey.Key;


            string method = null;
            if (args.Length == 5)
            {
                switch (args[0].ToLower())
                {
                    case "get": method = "get"; break;
                    case "put": method = "put"; break;
                }
            }

            if (method == null)
            {
                // display syntax if wrong number of parameters or unsupported operation
                Console.WriteLine ("FtpGetPut - uploads or downloads a file from a FTP server");
                Console.WriteLine ("Syntax: FtpGetPut get|put none|explicit|implicit hostname remotepath localpath ");
                Console.WriteLine ("Example: FtpGetPut get explicit test.rebex.net /pub/example/readme.txt readme.txt");
                return;
            }
            SslMode security = (SslMode) Enum.Parse (typeof(SslMode), args[1], true);
            string hostname = args[2];
            string remotePath = args[3];
            string localPath = args[4];

            // create Ftp object and connect to the server
            Ftp ftp = new Ftp();

            // set SSL to accept all server certificates...
            // do not do this in production code, server certificates should
            // be verified - use ftp.ValidatingCertificate event instead
            ftp.Settings.SslAcceptAllCertificates = true;
            
            ftp.Connect (hostname, security);
            // ask for username and password and login
            Console.Write ("Username: ");
            string username = Console.ReadLine();
            Console.Write ("Password: ");
            string password = Console.ReadLine();
            ftp.Login (username, password);

            // transfer the file
            long bytes;
            if (method == "get")
                bytes = ftp.GetFile (remotePath, localPath);
            else
                bytes = ftp.PutFile (localPath, remotePath);

            Console.WriteLine ("Transferred {0} bytes.", bytes);

            // disconnect from the server
            ftp.Disconnect();
        }
    }
}
