//  
//   Rebex Sample Code License
// 
//   Copyright 2024, REBEX CR s.r.o.
//   All rights reserved.
//   https://www.rebex.net/
// 
//   Permission to use, copy, modify, and/or distribute this software for any
//   purpose with or without fee is hereby granted.
// 
//   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//   OTHER DEALINGS IN THE SOFTWARE.
// 

using Rebex.Net;
using System;
using System.Net.Sockets;
using System.Threading.Tasks;

namespace Rebex.Samples
{
    public class TlsSocketDataProvider
    {
        public TlsSocket Socket { get; private set; }

        public ArraySegment<byte> Buffer { get; private set; }

        public TlsSocketDataProvider(TlsSocket socket, int bufferSize)
        {
            Socket = socket;
            Buffer = new ArraySegment<byte>(new byte[bufferSize]);
        }

        public virtual Task<int> ReceiveAsync()
        {
            return Socket.ReceiveAsync(Buffer);
        }

        public virtual async Task SendAsync(ArraySegment<byte> buffer)
        {
            while (buffer.Count > 0)
            {
                int n = await Socket.SendAsync(buffer).ConfigureAwait(false);
                buffer = ArraySegmentHelper.Slice(buffer, n);
            }
        }

        public virtual async Task CloseAsync()
        {
            try { await Socket.ShutdownAsync().ConfigureAwait(false); } catch { }
            await Socket.DisposeAsync().ConfigureAwait(false);
        }
    }
}
